#!/bin/sh
# ovirt-engine-log-setup-event.sh - log an event during setup/upgrade/cleanup
#
# It logs to a file which should not rotate and allow getting a clear view of
# the entire line of versions installed/upgraded/cleaned/etc.
#
# Copyright (C) 2013 Red Hat, Inc.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#

usage() {
	cat << __EOF__
Usage: $0
	--notes=NOTES
	Add NOTES to the log

__EOF__
}

notes=""

while [ -n "$1" ]; do
	x="$1"
	v="${x#*=}"
	shift
	case "${x}" in
		--notes=*)
			notes="${v}"
			;;
		--help)
			usage
			exit 0
			;;
		*)
			echo "Invalid option '${x}'"
			usage
			exit 1
			;;
	esac
done

get_distro_name() {
	python -c 'import platform; print (platform.linux_distribution(full_distribution_name=0)[0])'
}

get_distro_packages() {
	local distro="$(get_distro_name)"

	case "${distro}" in
		redhat|centos|fedora|SuSE)
			LC_ALL=C rpm -qa --last
			;;
		debian|ubuntu)
			dpkg -l | awk '/^ii/ {print $2, $3}'
			;;
		*)
			echo "$0: distribution ${distro} not supported"
			;;
	esac
}

prefix_cmd(){
	local prefix="$1"
	shift
	"$@" 2>&1 | sed "s/^/${prefix}: /"
}

(
	echo SETUP-EVENT-BEGIN
	prefix_cmd DATE date +'%Y-%m-%d %H:%M:%S'
	prefix_cmd NOTES echo "${notes}"
	prefix_cmd ENGINE_CFG engine-config -g ProductRPMVersion
	prefix_cmd ENGINE_CFG engine-config -g VdcVersion
	prefix_cmd JBOSS_VER jboss-cli --version
	prefix_cmd VIRSH_VER virsh --version
	prefix_cmd UNAME_A uname -a
	prefix_cmd PACKAGE get_distro_packages | sort
	echo SETUP-EVENT-END
) >> /var/lib/ovirt-engine/setup-history.txt
